﻿AuraCache = LibStub("AceAddon-3.0"):NewAddon("AuraCache", "AceConsole-3.0", "AceEvent-3.0")

-- lokale Variablen
local GetTime, pairs, ipairs = GetTime, pairs, ipairs
local UnitGUID = UnitGUID
local sfind, ssub = string.find, string.sub
local bitband = bit.band

local lastGC = 0 -- Zeitpunkt der letzten Garbage Collection

-- Folgende Tabellen haben GUID als Schlüssel
local LastSeen = {}
local LastAuraScan = {}	
local Buff_ByGUID = {}
local Debuff_ByGUID = {}

-- dynamische Whitelist für GUIDs außerhalb des Raids / Party
local Whitelist = {}

-- Folgende Tabellen haben UnitID als Schlüssel
local LastAuraNotify = {}
local ScheduledRescan = {}
local InstantRescan = {}

-- Listener für Änderungen
local listener = {}
local listenerScan = {}

-- time-Offset
local timeOffset = 0

function AuraCache:OnInitialize()
	self.NILGUID = "0x0000000000000000"
end

function AuraCache:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_AURA")
end 

function AuraCache:OnDisable()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UnregisterEvent("UNIT_AURA")
end 

function AuraCache:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
	local spellId, auraType, doseAmount

	-- Zunächst filtern, ob wir für diese GUID überhaupt die Auren cachen wollen
	local fails = true
	if Whitelist[destGUID] then
		fails = false
	elseif destFlags then
		local affiliation = bitband(destFlags, COMBATLOG_OBJECT_AFFILIATION_MASK)
		if affiliation ~= COMBATLOG_OBJECT_AFFILIATION_OUTSIDER then
			fails = false
		end
		if fails and bitband(destFlags, COMBATLOG_OBJECT_FOCUS) > 0 then
			fails = false
		elseif fails and bitband(destFlags, COMBATLOG_OBJECT_TARGET) > 0 then
			fails = false
		end
	end

	if fails then return end

	if type == "SPELL_AURA_APPLIED" or type == "SPELL_AURA_REMOVED" or type == "SPELL_AURA_REFRESH" or type == "SPELL_AURA_BROKEN" then
		spellId, _, _, auraType = ...
    
		self:COMBAT_LOG_EVENT(timestamp, type, sourceGUID, destGUID, spellId, auraType, 1)
	elseif type == "SPELL_AURA_APPLIED_DOSE" or type == "SPELL_AURA_REMOVED_DOSE" then
		spellId, _, _, auraType, doseAmount = ...
   
		self:COMBAT_LOG_EVENT(timestamp, type, sourceGUID, destGUID, spellId, auraType, doseAmount)
	elseif type == "SPELL_AURA_BROKEN_SPELL" then
		spellId, _, _, _, _, _, auraType = ...
		
		self:COMBAT_LOG_EVENT(timestamp, type, sourceGUID, destGUID, spellId, auraType, 0)
	end
	self:GarbageCollect()
end

function AuraCache:UNIT_AURA(msg, arg1)
	self:UpdateUnitAura(arg1)
	self:GarbageCollect()
	self:Rescan()
end

function AuraCache:GetTimeOffset()
	return timeOffset
end

function AuraCache:GetDebuffType(spellId) 
	return CSL_Factory:GetDebuffType(spellId)
end

function AuraCache:Rescan()
	local now = GetTime()
	for k, v in pairs(ScheduledRescan) do
		if v > 0 and v < now then
			self:UpdateUnitAura(k)
		end		
	end
end

-- Müllentsorgung
function AuraCache:GarbageCollect()
	local now = GetTime()
	if lastGC > now - 60 then return end
	lastGC = now

	local timeLimit = now - 120
	for k, v in pairs(LastSeen) do
		if v < timeLimit then
			LastSeen[k] = nil
			Buff_ByGUID[k] = nil
			Debuff_ByGUID[k] = nil
			Whitelist[k] = nil
		end
	end
end

-- Initialisierung der Datenstruktur pro GUID
function AuraCache:CreateInfoTab(GUID, auraType)
	local infotab = {}

	infotab.GUID = GUID
	infotab.auraType = auraType -- BUFF, DEBUFF
	infotab.num = 0	-- Anzahl Auren	
	infotab.numSpellId = {} -- spellId => integer
	infotab.numMagic = 0    -- Anzahl Magie Debuffs
	infotab.numPoison = 0   -- Anzahl Gift Debuffs
	infotab.numDisease = 0  -- Anzahl Krankheit Debuffs
	infotab.numCurse = 0    -- Anzahl Fluch Debuffs

	infotab.spellId = {}    -- SpellId
	infotab.caster = {}     -- Caster GUID
	infotab.count = {}		-- Anzahl Stacks
	infotab.duration = {}		-- Dauer der Aura
	infotab.expirationTime = {}	-- Ende der Aura
	
	infotab.sorted = {}     -- Sortiertabelle
		
	return infotab
end

function AuraCache:InitGUID(GUID)
	LastSeen[GUID] = GetTime()
	Buff_ByGUID[GUID] = self:CreateInfoTab(GUID, "BUFF")
	Debuff_ByGUID[GUID] = self:CreateInfoTab(GUID, "DEBUFF")

	return Buff_ByGUID[GUID], Debuff_ByGUID[GUID]
end

function AuraCache:GetInfoTab(GUID, auraType)
	if not GUID then return end
	if Buff_ByGUID[GUID] == nil then
		self:InitGUID(GUID)
	end

	if auraType == "BUFF" then
		return Buff_ByGUID[GUID]
	elseif auraType == "DEBUFF" then
		return Debuff_ByGUID[GUID]
	else
		return Buff_ByGUID[GUID], Debuff_ByGUID[GUID]	
	end
end

function AuraCache:GetSortedAuras(GUID, auraType)
	local infotab = self:GetInfoTab(GUID, auraType)

	local sorted = infotab.sorted
	for i = 1, infotab.num do
		sorted[i] = i
	end
	for i = infotab.num + 1, #sorted do
		sorted[i] = nil
	end
	table.sort(infotab.sorted, 
		function(a, b) 
			local aNull = (infotab.expirationTime[a] or 0) == 0
			local bNull = (infotab.expirationTime[b] or 0) == 0
			if aNull == bNull then
				return infotab.expirationTime[a] < infotab.expirationTime[b]
			elseif aNull then
				return false
			else
				return true
			end
		end
	)

	return infotab
end

function AuraCache:ResetInfoTab(infotab)
	infotab.num = 0
	infotab.numMagic = 0 
	infotab.numPoison = 0
	infotab.numDisease = 0
	infotab.numCurse = 0
	infotab.numSpellId = wipe(infotab.numSpellId)
end

-- Fügt neue Aura in eine InfoTab ein.
function AuraCache:Insert(infotab, index, spellId, caster, count, duration, expirationTime)
	LastSeen[infotab.GUID] = GetTime()	
	
	local numSpells = infotab.numSpellId[spellId] or 0
	
	if (index or 0) == 0 then
		index = infotab.num + 1
	end
	infotab.num = index
	infotab.spellId[index] = spellId
	infotab.caster[index] = caster
	infotab.count[index] = count
	infotab.duration[index] = duration
	infotab.expirationTime[index] = expirationTime
				
	infotab.numSpellId[spellId] = numSpells + 1
		
	local curDebuff = CSL_Factory:GetDebuffType(spellId)
	if curDebuff == "Magic" then
		infotab.numMagic = (infotab.numMagic or 0) + 1
	elseif curDebuff == "Disease" then
		infotab.numDisease = (infotab.numDisease or 0) + 1
	elseif curDebuff == "Poison" then 
		infotab.numPoison = (infotab.numPoison or 0) + 1
	elseif curDebuff == "Curse" then 
		infotab.numCurse = (infotab.numCurse or 0) + 1
	end	
end

-- Entfernt eine Aura aus einer InfoTab. Gibt true zurück, falls erfolgreich sonst false
function AuraCache:Remove(infotab, spellId)
	LastSeen[infotab.GUID] = GetTime()

	infotab.numSpellId[spellId] = math.max(0, (infotab.numSpellId[spellId] or 0) - 1)
		
	local curDebuff = CSL_Factory:GetDebuffType(spellId)
	if curDebuff == "Magic" then 
		infotab.numMagic = (infotab.numMagic or 1) - 1
	elseif curDebuff == "Disease" then
		infotab.numDisease = (infotab.numDisease or 1) - 1 
	elseif curDebuff == "Poison" then 
		infotab.numPoison = (infotab.numPoison or 1) - 1
	elseif curDebuff == "Curse" then
		infotab.numCurse = (infotab.numCurse or 1) - 1
	end		
end

function AuraCache:RegisterListener(event, callback)
	listener[callback] = event
end

function AuraCache:RegisterListenerScan(callback)
	listenerScan[callback] = true
end

function AuraCache:Notify(event, timestamp, infotab, spellId, caster, amount)	
	for k, v in pairs(listener) do
		if v == event or v == "ANY" then
			k(event, timestamp, infotab.GUID, infotab.auraType, CSL_Factory:GetDebuffType(spellId),
			spellId, caster, amount, 0, 0)
		end
	end
end

function AuraCache:NotifyScan(unitId)
	for k, v in pairs(listenerScan) do
		if v then
			k(unitId)
		end
	end
end

function AuraCache:RegisterInstantRescan(unitId)
	InstantRescan[unitId] = true
end

-- Update durch COMBAT_LOG_EVENT_UNFILTERED
function AuraCache:COMBAT_LOG_EVENT(timestamp, event, sourceGUID, destGUID, spellId, auraType, amount)
	timeOffset = timestamp - time()
	
	if event == "SPELL_AURA_APPLIED" then
		self:SPELL_AURA_APPLIED(timestamp, sourceGUID, destGUID, spellId, auraType, 1)
	elseif event == "SPELL_AURA_REMOVED" or event == "SPELL_AURA_BROKEN" or event == "SPELL_AURA_BROKEN_SPELL" then
		self:SPELL_AURA_REMOVED(timestamp, sourceGUID, destGUID, spellId, auraType, 1)
	elseif event == "SPELL_AURA_REFRESH" then
		self:SPELL_AURA_REFRESH(timestamp, sourceGUID, destGUID, spellId, auraType, 1)
	elseif event == "SPELL_AURA_APPLIED_DOSE" then
		self:SPELL_AURA_APPLIED_DOSE(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	elseif event == "SPELL_AURA_REMOVED_DOSE" then
		self:SPELL_AURA_REMOVED_DOSE(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	end
end

function AuraCache:SPELL_AURA_APPLIED(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	local infotab = self:GetInfoTab(destGUID, auraType)	

	local duration = CSL_Factory:GetSpellDuration(spellId) or 0
	local expirationTime = GetTime() + duration
			
	self:Insert(infotab, nil, spellId, sourceGUID, amount, duration, expirationTime)
	self:Notify("SPELL_AURA_APPLIED", timestamp, infotab, spellId, sourceGUID, amount)
end

function AuraCache:SPELL_AURA_REMOVED(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	local infotab = self:GetInfoTab(destGUID, auraType)
	
	self:Remove(infotab, spellId, sourceGUID)
	self:Notify("SPELL_AURA_REMOVED", timestamp, infotab, spellId, sourceGUID, amount)
end

function AuraCache:SPELL_AURA_REFRESH(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	local infotab = self:GetInfoTab(destGUID, auraType)

	self:Notify("SPELL_AURA_REFRESH", timestamp, infotab, spellId, sourceGUID, amount)
end

function AuraCache:SPELL_AURA_APPLIED_DOSE(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	local infotab = self:GetInfoTab(destGUID, auraType)
				
	self:Notify("SPELL_AURA_APPLIED_DOSE", timestamp, infotab, spellId, sourceGUID, amount)
end

function AuraCache:SPELL_AURA_REMOVED_DOSE(timestamp, sourceGUID, destGUID, spellId, auraType, amount)
	local infotab = self:GetInfoTab(destGUID, auraType)
	
	self:Notify("SPELL_AURA_REMOVED_DOSE", timestamp, infotab, spellId, sourceGUID, amount)			
end

-- Update durch UnitBuff/UnitDebuff
function AuraCache:UpdateUnitAura(unitId)
	local scanInterval
	local now = GetTime()
	local GUID = UnitGUID(unitId)
	if GUID == nil then return end -- Safeguard	
	
	-- Einheiten für die ein UNIT_AURA Event kam, werden gewhitelisted
	Whitelist[GUID] = true	

	if InstantRescan[unitId] then
		scanInterval = 0.2
	else
		scanInterval = 1.0
	end

	-- Scan Throttle
	local lastScan = LastAuraScan[GUID] or 0
	if lastScan > now - scanInterval then -- and not InstantRescan[unitId] then 
		if ScheduledRescan[unitId] or 0 == 0 then
			ScheduledRescan[unitId] = now + scanInterval
		end 
		if (LastAuraNotify[unitId] or 0) < now - scanInterval then
			self:NotifyScan(unitId)
			LastAuraNotify[unitId] = now
		end
		return
	else
		ScheduledRescan[unitId] = 0
	end 
	LastAuraScan[GUID] = now
	LastAuraNotify[unitId] = now
			
	local bufftab, debufftab = self:GetInfoTab(GUID)
	self:ResetInfoTab(bufftab)
	self:ResetInfoTab(debufftab)
	
	local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
	local i = 1
		
	-- Buffs auslesen
	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(unitId, i)
	while name do
		if (count or 0) < 1 then count = 1 end
		if unitCaster ~= nil then unitCaster = UnitGUID(unitCaster) end
		if not unitCaster then unitCaster = self.NILGUID end
				
		CSL_Factory:UpdateSpellInfo(spellId, name, icon, duration, debuffType)

		self:Insert(bufftab, i, spellId, unitCaster, count, duration, expirationTime)

		i = i + 1
		name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(unitId, i)		
	end

	-- Debuffs auslesen
	i = 1
	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(unitId, i)
	while name do
		if (count or 0) < 1 then count = 1 end
		if unitCaster ~= nil then unitCaster = UnitGUID(unitCaster) end
		if not unitCaster then unitCaster = self.NILGUID end

		CSL_Factory:UpdateSpellInfo(spellId, name, icon, duration, debuffType)

		self:Insert(debufftab, i, spellId, unitCaster, count, duration, expirationTime)
			
		i = i + 1
		name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(unitId, i)		
	end

	self:NotifyScan(unitId)
end

-- Typ der Aura: Magic, Disease, Poison, Curse
function AuraCache:HasDebuffType(GUID, debuffType)
	local debufftab = self:GetInfoTab(GUID, "DEBUFF")
	
	if debuffType == "Magic" then
		return (debufftab.numMagic or 0) > 0
	elseif debuffType == "Poison" then
		return (debufftab.numPoison or 0) > 0
	elseif debuffType == "Disease" then
		return (debufftab.numDisease or 0) > 0
	elseif debuffType == "Curse" then
		return (debufftab.numCurse or 0) > 0
	else
		return nil
	end
end

function AuraCache:HasAura(GUID, auraType, primary, secondary, tertiary)
	local infotab = self:GetInfoTab(GUID, auraType)
	
	local primaryCount = (infotab.numSpellId[primary] or 0)
	local secondaryCount = (infotab.numSpellId[secondary] or 0)
	local tertiaryCount = (infotab.numSpellId[tertiary] or 0)
	
	return primaryCount > 0 or secondaryCount > 0 or tertiaryCount > 0
end

function AuraCache:HasAuraName(GUID, auraType, spellName)
	if self:FindFirst(GUID, auraType, spellName) then
		return true
	else
		return false
	end
end

function AuraCache:FindFirst(GUID, auraType, spellName)
	local infotab = self:GetInfoTab(GUID, auraType)

	for i = 1, infotab.num do
		local name = CSL_Factory:GetSpellName(infotab.spellId[i])

		if spellName == name then
			return i
		end
	end	
end