﻿CSL_Factory = {}

local GetSpellInfo, pairs = GetSpellInfo, pairs

local spellNameCache = {}
local spellIconCache = {}
local spellDurationCache = {}
local debuffTypeCache = {}

function CSL_Factory:CreateSpellList(MixedSpellList, isOrderedList)
	local sl = {
		cache = {},
		mixedList = MixedSpellList,
		isOrderedList = isOrderedList,
	}
	return sl
end

function CSL_Factory:ResetCache(sl)
	sl.cache = {}
end

function CSL_Factory:GetSpellName(spellId)
	local name = spellNameCache[spellId]
	local icon = spellIconCache[spellId]
	if name == nil and (spellId or 0) > 0 then
		name, _, icon = GetSpellInfo(spellId)
		spellNameCache[spellId] = name
		spellIconCache[spellId] = icon
	end
	return name, icon
end

function CSL_Factory:UpdateSpellInfo(spellId, spellName, icon, duration, debuffType)
	if (spellId or 0) > 0 then
		spellNameCache[spellId] = spellName
		spellIconCache[spellId] = icon
		spellDurationCache[spellId] = duration
		debuffTypeCache[spellId] = debuffType
	end
end

function CSL_Factory:GetSpellInfo(spellId)
	local name = spellNameCache[spellId]
	if name == nil and (spellId or 0) > 0 then
		name, _, spellIconCache[spellId] = GetSpellInfo(spellId)
		spellNameCache[spellId] = name
	end
	return name, spellIconCache[spellId], spellDurationCache[spellId], debuffTypeCache[spellId]
end

function CSL_Factory:GetDebuffType(spellId)
	return debuffTypeCache[spellId]
end

function CSL_Factory:GetSpellDuration(spellId)
	return spellDurationCache[spellId]
end

function CSL_Factory:IsOnSpellList(sl, spellId)
	local isOnList = sl.cache[spellId]
	if isOnList ~= nil then
		return isOnList
	else	
		local spellName = self:GetSpellName(spellId)

		if sl.isOrderedList then
			for k, v in pairs(sl.mixedList) do
				if tonumber(v) then
					if tonumber(v) == spellId then
						sl.cache[spellId] = k
						return k
					end
				else
					if v == spellName then
						sl.cache[spellId] = k
						return k
					end
				end		
			end
			sl.cache[spellId] = false
			return false
		else
			for k, v in pairs(sl.mixedList) do
				if v then
					if tonumber(k) then
						if k == spellId then
							sl.cache[spellId] = true
							return true
						end
					else
						if k == spellName then
							sl.cache[spellId] = true
							return true
						end
					end
				end
			end
			sl.cache[spellId] = false
			return false
		end
	end
end