﻿-- BdwOptions

local newbarname
function BdwAddon:CreateBaseOptions()
	self.options = {
		type = "group",
		args = {
			locked = {
				order = 0,
				name = "Locked",
				desc = "Lock / unlock the bars",
				type = "toggle",
				set = function(info,val) self.db.char.barslocked = val; self:EnableAllDragable(not self.db.char.barslocked) end,
				get = function(info) return self.db.char.barslocked end
			},
			base = {
				order = 1,
				name = "Base options",
				desc = "Configure base options",
				type = "group",
				args = {
					tooltips = {
						name = "Show Tooltips",
						desc = "Show Buff/Debuff Tooltips",
						type = "toggle",
						set = function(info,val) self.db.char.tooltips = val end,
						get = function(info) return self.db.char.tooltips end
					},
					cooldownalpha = {
						name = "Cooldown model opacity",
						type = "range",
						min = 0,
						max = 100,
						step = 1,
						bigStep = 10,
						set = function(info,val) self.db.char.cooldownalpha = val end,
						get = function(info) return self.db.char.cooldownalpha or 100 end
					},
				},
			},
			bars = {
				order = 2,
				name = "Bars",
				desc = "Configure the bars",
				type = "group",
				args = {
					barname = {
						name = "Bar Name",
						desc = "Enter Name for new Bar",
						type = "input",   
						confirm = false,     
						set = function(info,val) newbarname = val end,
						get = function(info) return newbarname end
					},
					cmdaddbar = {
						name = "Add Bar",
						type = "execute",
						func = function()
							if newbarname then
								self:AddBar(newbarname)
								self:CreateBarOptions(newbarname)
							end
						end,
					},
					cmdremovebar = {
						name = "Remove Bar",
						type = "execute",
						func = function() self:RemoveBar(newbarname) end,
					},
				}
			}
		}   
	}
end

function BdwAddon:CreateBarOptions(barname)
	self.options.args.bars.args[barname] = {
		name = barname,
		type = "group",
		args = {
			hdrLayout = {
				order = 0,
				name = "Layout",
				type = "header",
			},				
			direction = {
				order = 1,
				name = "Icon direction",
				desc = "Direction of icon placement",
				type = "select",
				values = { [1] = "Right", [2] = "Left", [3] = "Up", [4] = "Down" },
				set = function(info,val) self.db.char.bars[info[#info-1]].direction = val; BdwAddon:UpdateFrameSize(info[#info-1]) end,
				get = function(info) return self.db.char.bars[info[#info-1]].direction end
			},
			basesize = {
				order = 2,
				name = "Base Size",
				desc = "Size of Aura Icons",
				type = "range",
				min = 0,
				max = 128,
				step = 1,
				bigStep = 8,
				set = function(info,val) self.db.char.bars[info[#info-1]].basesize = val; BdwAddon:UpdateFrameSize(info[#info-1]) end,
				get = function(info) return self.db.char.bars[info[#info-1]].basesize end
			},
			Count = {
				order = 4,
				name = "Count",
				desc = "Options for applied stack count",
				type = "group",
				inline = true,
				args = {
					CountJustifyV = {
						order = 0,
						name = "Vertical Alignment",
						desc = "Choose Vertical Alignment",
						type = "select",   
						confirm = false,     
						values = { ["TOP"] = "Top", ["MIDDLE"] = "Middle", ["BOTTOM"] = "Bottom", },
						get = function(info)
							return (self.db.char.bars[info[#info-2]].CountJustifyV or "BOTTOM")
						end,
						set = function(info,val)
							self.db.char.bars[info[#info-2]].CountJustifyV = val
							self:UpdateFrameSize(info[#info-2])
						end,						
					},									
					CountJustifyH = {
						order = 1,
						name = "Horizontal Alignment",
						desc = "Choose Horizontal Alignment",
						type = "select",   
						confirm = false,     
						values = { ["LEFT"] = "Left", ["CENTER"] = "Center", ["RIGHT"] = "Right", },
						get = function(info)
							return (self.db.char.bars[info[#info-2]].CountJustifyH or "RIGHT")
						end,
						set = function(info,val)
							self.db.char.bars[info[#info-2]].CountJustifyH = val;
							self:UpdateFrameSize(info[#info-2])
						end,						
					},									
					CountFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Count font",
						desc = "Font uses for the applied stack count text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.char.bars[info[#info-2]].CountFont or "Friz Quadrata TT")
						end,
						set = function(info, val)
							self.db.char.bars[info[#info-2]].CountFont = val
							self:UpdateFrameSize(info[#info-2])
						end,
					},
					CountFontSize = {
						order = 3,
						name = "Count font size",
						desc = "Choose count font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.char.bars[info[#info-2]].CountFontSize or 11)
						end,
						set = function(info,val) 
							self.db.char.bars[info[#info-2]].CountFontSize = val
							self:UpdateFrameSize(info[#info-2])
						end,						
					},
					CountColor = {
						order = 4,
						name = "Font color",
						desc = "Define the count font color",
						type = "color",
						hasAlpha = true,
						get = function(info)
							local baropt = self.db.char.bars[info[#info-2]]
							return baropt.CountR or 1, baropt.CountG or 1, baropt.CountB or 1, baropt.CountA or 1
						end,
						set = function (info, r, g, b, a)							
							local baropt = self.db.char.bars[info[#info-2]]
							baropt.CountR = r
							baropt.CountG = g
							baropt.CountB = b
							baropt.CountA = a							
							self:UpdateFrameSize(info[#info-2])
						end
					},
					CountFontOutline = {
						order = 5,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.char.bars[info[#info-2]].CountFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.char.bars[info[#info-2]].CountFontFlags = "OUTLINE"
							else
								self.db.char.bars[info[#info-2]].CountFontFlags = ""
							end		
							self:UpdateFrameSize(info[#info-2])					
						end,						
					},
				},
			},
			hdrBuffs = {
				order = 5,
				name = "Buffs",
				type = "header",
			},				
			showbuffs = {
				order = 6,
				name = "Consider Buffs",
				desc = "Consider the buffs for inclusion in the bar",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showbuffs = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showbuffs end
			},
			showallbuffs = {
				order = 7,
				name = "Show all buffs",
				desc = "Show all buffs regardless who applied them",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showallbuffs = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showallbuffs end
			},     
			showplayerbuff = {
				order = 8,
				name = "Show player buffs",
				desc = "Show all buffs applied by the player",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showplayerbuff = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showplayerbuff end
			},
			showenemybuff = {
				order = 9,
				name = "Show buffs of enemies",
				desc = "Show all buffs of enemies",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showenemybuff = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showenemybuff end        
			},
			hdrDebuffs = {
				order = 10,
				name = "Debuffs",
				type = "header",
			},				
			showdebuffs = {
				order = 11,
				name = "Consider Debuffs",
				desc = "Consider the buffs for inclusion in the bar",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showdebuffs = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showdebuffs end
			},
			showcaster = {
				order = 12,
				name = "Show caster debuffs",
				desc = "Show all debuffs relevant to casters",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showcaster = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showcaster end
			},
			showtank = {
				order = 13,
				name = "Show tank debuffs",
				desc = "Show all debuffs relevant to tanks",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showtank = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showtank end
			},
			onlyindi = {
				order = 14,
				name = "Show only indicator",
				desc = "Show only indicator for missing debuff groups",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].onlyindi = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].onlyindi end
			},
			showalldebuffs = {
				order = 15,
				name = "Show all debuffs",
				desc = "Show all debuffs regardless who applied them",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showalldebuffs = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showalldebuffs end
			},     
			showplayer = {
				order = 16,
				name = "Show player debuffs",
				desc = "Show all debuffs applied by the player",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showplayer = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showplayer end
			},
			showcombo = {
				order = 17,
				name = "Show Combopoints",
				desc = "Show the number of combo points on the current target",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showcombo = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showcombo end
			},
			hdrMisc = {
				order = 20,
				name = "Misc",
				type = "header",
			},				
			showauras = {
				order = 21,
				name = "Show Auras",
				desc = "Show Aura-style buffs",
				type = "toggle",
				set = function(info,val) self.db.char.bars[info[#info-1]].showauras = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].showauras end
			},
			duration = {
				order = 22,
				name = "Duration treshold",
				desc = "Only display auras with duration less than x seconds (0 = display all)",
				type = "range",
				min = 0,
				max = 120,
				step = 1,
				bigStep = 10,
				set = function(info,val) self.db.char.bars[info[#info-1]].duration = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].duration end       
			},
			unitid = {
				order = 23,
				name = "Unit ID",
				desc = "Specifies the unit ID",
				type = "input",
				confirm = false,
				set = function(info,val) self.db.char.bars[info[#info-1]].unitid = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].unitid end        
			},
			ignorelist = {
				order = 24,
				name = "Ignore List",
				desc = "Specify Aura's Names to ignore (separated by ,)",
				type = "input",
				confirm = false,
				multiline = true,
				set = function(info,val) self.db.char.bars[info[#info-1]].ignorelist = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].ignorelist end
			},
			includelist = {
				order = 25,
				name = "Include List",
				desc = "Specify Aura's Names to always include (separated by ,)",
				type = "input",
				confirm = false,
				multiline = true,
				set = function(info,val) self.db.char.bars[info[#info-1]].includelist = val end,
				get = function(info) return self.db.char.bars[info[#info-1]].includelist end
			}
		},
	}
end