BdwAddon = LibStub("AceAddon-3.0"):NewAddon("BDW2", "AceConsole-3.0", "AceEvent-3.0")

local bars = {}
local splnames = {}
local newbarname

local red_icon = 42994
local yellow_icon = 42993
local green_icon = 42992
local combo_icon = {
	"Interface\\AddOns\\BDW2\\Images\\ComboPoints\\Blue.tga",
	"Interface\\AddOns\\BDW2\\Images\\ComboPoints\\Green.tga",
	"Interface\\AddOns\\BDW2\\Images\\ComboPoints\\Yellow.tga",
	"Interface\\AddOns\\BDW2\\Images\\ComboPoints\\Orange.tga",
	"Interface\\AddOns\\BDW2\\Images\\ComboPoints\\Red.tga",
}

local buffdebuffgroups = {
	"Agility and Strength",
	"Armor (%)",
	"Attack Power",
	"Attack Power (%)",
	"Bloodlust/Heroism",
	"Damage (%)",
	"Damage Reduction (%)",
	"Haste (%)",
	"Healing Received (%)",
	"Health",
	"Intellect",
	"Mana Regen (Blessing)",
	"Melee Critical Strike Chance",
	"Melee Haste",
}

local agnames = {
	"Spell Damage",
	"Spell Crit Chance",
	nil,
	nil,
	"Attack Power",
	"Speed Slow",
	nil,
	nil,
	"Armor (Major)",
	"Armor (Minor)",
}

local defaults = {
    char = {
        barslocked = true,
        bars = {},
    },
}

function BdwAddon:OnInitialize()
	-- Code that you want to run when the addon is first loaded goes here.
  
	local acedb = LibStub("AceDB-3.0")
	self.acecfg = LibStub("AceConfig-3.0")
	self.acecd = LibStub("AceConfigDialog-3.0")
	self.LSM = LibStub("LibSharedMedia-3.0")

	self.db = acedb:New("BdwDB", defaults, "Default")

	for k in pairs(self.db.char.bars) do
		self:AddBar(k)
	end  

	self:RegisterChatCommand("bdw", "OnSlash")	

	self:FillSplNames()

	for k in pairs(bars) do
		self:GetMineAuras(k)
		self:DisplayAuras(k)
	end

	hooksecurefunc(GameTooltip, "SetUnitAura", hook_SetUnitAura)
	hooksecurefunc(GameTooltip, "SetUnitBuff", hook_SetUnitBuff)
	hooksecurefunc(GameTooltip, "SetUnitDebuff", hook_SetUnitDebuff)
end

function BdwAddon:AddBar(barname)
	if not barname then return end

	if not self.db.char.bars[barname] then
		self.db.char.bars[barname] = {
			point = "CENTER",
			relativePoint = "CENTER",
			x = 0,
			y = 0,
			basesize = 32,      
			direction = 1,
			showbuffs = true,
			showdebuffs = true,
			showcaster = false, 
			showtank = false,
			onlyindi = false,
			showplayerbuff = false,
			showplayer = true,
			showauras = false,
			duration = 0,   
			unitid = "target",
			ignorelist = "",
			includelist = "",
		}    
	end

	local baropt = self.db.char.bars[barname]

	baropt.point = baropt.point or "CENTER"
	baropt.relativePoint = baropt.relativePoint or "CENTER"
	baropt.unitid = baropt.unitid or "target"
	baropt.ignorelist = baropt.ignorelist or ""
	baropt.includelist = baropt.includelist or ""
	baropt.basesize = baropt.basesize or 32
	baropt.direction = baropt.direction or 1

	bars[barname] = {
		frame = self:CreateFrame(barname, baropt),
		afr = {},

		active = { false, false, false, false, false, false, false, false, },
		spellIds = { 0, 0, 0, 0, 0, 0, 0, 0, },
		counts = { 0, 0, 0, 0, 0, 0, 0, 0, },
		durations = { 0, 0, 0, 0, 0, 0, 0, 0, },
		expirationTimes = { 0, 0, 0, 0, 0, 0, 0, 0, },
		reverses = { false, false, false, false, false, false, false, false, },
		isbuffs = { false, false, false, false, false, false, false, false, },
		buffindexs = { 0, 0, 0, 0, 0, 0, 0, 0, },
		tooltipTexts = { "", "", "", "", "", "", "", "", },    

		fauras = {},
		auragroups = {},
	}

	AuraCache:RegisterInstantRescan(baropt.unitid)
end

function BdwAddon:RemoveBar(barname)
	if not barname then return end
	if bars[barname] then
		bars[barname].frame:UnregisterAllEvents()
		bars[barname].frame:EnableMouse(false)
		bars[barname].frame:Hide()
	end
	self.db.char.bars[barname] = nil
	bars[barname] = nil
	self.options.args.bars.args[barname] = nil
end

function BdwAddon:FillSplNames()
	splnames = {
		caster = {
			-- Mond und Erde
			[60433] = 1,
			-- Meister der Gifte
			[93068] = 1,
			-- Schwarze Seuche
			[65142] = 1,
			-- Fluch der Elemente
			[1490] = 1,
			-- Kritische Masse
			[22959] = 2,
			-- Winters Chill
			[12579] = 2, --CHECK
			-- Shadow Mastery
			[17800] = 2,
		},
		tanks = {
			-- Demo shout
			[1160] = 5,
			-- Demo Roar
			[99] = 5,
			-- Curse of Weakness
			[702] = 5,
			-- Frost Fever
			[55095] = 6,
			-- Infected Wounds
			[58181] = 6, --CHECK
			-- Judgement of the Just
			[68055] = 6,
			-- Thunder Clap
			[6343] = 6,
			-- Vindication
			[26017] = 5,
			-- Sunder Armor
			[58567] = 9,
			-- Faerie fire
			[91565] = 9,
			-- Expose Armor
			[8647] = 9,
		},
	}
end

function BdwAddon:OnEnable()
    -- Called when the addon is enabled
	self:CreateBaseOptions()
	for k, _ in pairs(bars) do
		self:CreateBarOptions(k)
	end
	self.acecfg:RegisterOptionsTable("BDW", self.options)
	self.acecd:AddToBlizOptions("BDW")
	
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED")
	self:RegisterEvent("UNIT_PET")
	self:RegisterEvent("UNIT_COMBO_POINTS")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")

	AuraCache:RegisterListenerScan(function (unitId) self:AuraCacheNotifier(unitId) end)		
end

function BdwAddon:OnDisable()
    -- Called when the addon is disabled
	DEFAULT_CHAT_FRAME:AddMessage("BDW2 Disable")

	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
	self:UnregisterEvent("UNIT_PET")
	self:UnregisterEvent("UNIT_COMBO_POINTS")
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
end

function BdwAddon:DragStop(frm)
	frm:StopMovingOrSizing()

	local point, relativeTo, relativePoint, xOfs, yOfs
	point, relativeTo, relativePoint, xOfs, yOfs = frm:GetPoint()
	self.db.char.bars[frm.barname].point = point
	self.db.char.bars[frm.barname].relativePoint = relativePoint
	self.db.char.bars[frm.barname].x = xOfs
	self.db.char.bars[frm.barname].y = yOfs
end 

function BdwAddon:CreateFrame(barname, baroptions)
	local frm = CreateFrame("Frame", nil, UIParent)
	frm.obj = self
	frm.barname = barname
	frm:SetWidth(baroptions.basesize) 
	frm:SetHeight(baroptions.basesize) -- for your Texture
	frm:EnableMouse(false)
	frm:SetMovable(false)
	frm:SetFrameStrata("BACKGROUND")

	frm:SetAttribute("unit", baroptions.unitid)

	RegisterUnitWatch(frm)

	frm.bck = frm:CreateTexture()
	frm.bck:SetAllPoints(frm)
	frm.bck:SetTexture(1, 0, 0)
	if self.db.char.barslocked then
		frm.bck:Hide()
	else
		frm.bck:Show()
	end

	frm:SetScript("OnDragStart", function(this) if(this:IsMovable()) then this:StartMoving() end end)
	frm:SetScript("OnDragStop", function(this) this.obj:DragStop(this) end)

	frm:RegisterForDrag("LeftButton")
	frm:SetPoint(self.db.char.bars[barname].point, UIParent, self.db.char.bars[barname].relativePoint, self.db.char.bars[barname].x, self.db.char.bars[barname].y)
	frm:Show()

	return frm
end

function BdwAddon:OnEnter(this)
	if self.db.char.tooltips then
		GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
		GameTooltip.owner = this

		if this.tooltipText then
			GameTooltip:SetText(this.tooltipText, 1, 0, 0)			
		elseif (this.buffindex or 0) > 0 then
			if this.isbuff then
				GameTooltip:SetUnitBuff(this.unitid, this.buffindex)
			else
				GameTooltip:SetUnitDebuff(this.unitid, this.buffindex)
			end
		elseif this.spellId then
			GameTooltip:SetSpellByID(this.spellId)
		end
	end
end

function BdwAddon:OnLeave(this)
	if (self.db.char.tooltips) then  
		GameTooltip:Hide()
		GameTooltip.owner = nil
	end
end

function BdwAddon:OnMouseDown(this, mouseButton)
	if mouseButton == "LeftButton" then
		if IsShiftKeyDown() then
			if ChatFrame1EditBox:IsVisible() then
				if this.spellId then
					local text = GetSpellLink(this.spellId)
					ChatFrame1EditBox:Insert(text)
				end
			end 
		elseif IsAltKeyDown() then
			local frm = this:GetParent()
			local spellName = nil

			if (this.buffindex) then
				if (this.isbuff) then
					spellName = UnitBuff(this.unitid, this.buffindex)
				else
					spellName = UnitDebuff(this.unitid, this.buffindex)
				end
			end

			if spellName then
				self.db.char.bars[frm.barname].ignorelist = self.db.char.bars[frm.barname].ignorelist .. "\n" .. spellName
			end
		end
	end
end

function BdwAddon:CreateAuraIndicator(bar, baroptions, position, index)
	local cafr, cslot, ctex, ccooldown, ccount, color

	if (position > #bar.afr) then
		cafr = CreateFrame("Frame", nil, bar.frame)
		cafr.obj = self
		cafr:EnableMouse(true)
		cafr:RegisterForDrag("LeftButton")
		cafr:SetScript("OnEnter", function(this) this.obj:OnEnter(this) end)
		cafr:SetScript("OnLeave", function(this) this.obj:OnLeave(this) end)
		cafr:SetScript("OnMouseDown", function(this, mouseButton) this.obj:OnMouseDown(this, mouseButton) end)
		cafr:SetScript("OnDragStart", function(this) if(this:GetParent():IsMovable()) then this:GetParent():StartMoving() end end)
		cafr:SetScript("OnDragStop", function(this) this:GetParent().obj:DragStop(this:GetParent()) end)

		ctex = cafr:CreateTexture(nil, "BORDER")
		ctex:SetAllPoints(cafr)
		cafr.icon = ctex

		cslot = cafr:CreateTexture(nil, "OVERLAY")
		cslot:SetPoint("TOPLEFT", cafr, "TOPLEFT", -1, 0)
		cslot:SetWidth(baroptions.basesize + 2)
		cslot:SetHeight(baroptions.basesize)
		cslot:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")
		cslot:SetTexCoord(0.296875, 0.5703125, 0, 0.515625)
		--    cslot:SetAllPoints(cafr)
		cafr.slot = cslot

		ccount = cafr:CreateFontString(nil, "OVERLAY")
		ccount:SetPoint("TOPLEFT", cafr)
		ccount:SetPoint("BOTTOMRIGHT", cafr)
		ccount:SetJustifyV(baroptions.CountJustifyV or "BOTTOM")
		ccount:SetJustifyH(baroptions.CountJustifyH or "RIGHT")
		ccount:SetFont(self.LSM:Fetch(self.LSM.MediaType.FONT, (baroptions.CountFont or "Friz Quadrata TT")), baroptions.CountFontSize or 11, baroptions.CountFontFlags or "")
		ccount:SetTextColor(baroptions.CountR or 1, baroptions.CountG or 1, baroptions.CountB or 1, baroptions.CountA or 1)
		cafr.appl = ccount

		ccooldown = CreateFrame("Cooldown", nil, cafr, "CooldownFrameTemplate")  
		ccooldown:SetAllPoints(ctex)
		cafr.cool = ccooldown

		bar.afr[position] = cafr
	else
		cafr = bar.afr[position]
	end 
	cafr:SetWidth(baroptions.basesize)
	cafr:SetHeight(baroptions.basesize)
	if (baroptions.direction == 1) then
		cafr:SetPoint("TOPLEFT", bar.frame, "TOPLEFT", (position - 1) * baroptions.basesize, 0)
	elseif (baroptions.direction == 2) then
		cafr:SetPoint("TOPLEFT", bar.frame, "TOPRIGHT", position * baroptions.basesize * (-1), 0)
	elseif (baroptions.direction == 3) then
		cafr:SetPoint("TOPLEFT", bar.frame, "TOPLEFT", 0, (position - 1) * baroptions.basesize)
	else
		cafr:SetPoint("TOPLEFT", bar.frame, "TOPLEFT", 0, (position - 1) * baroptions.basesize * (-1))
	end
	
	local icon
	local debuffType
	if tonumber(bar.spellIds[index]) then
		_, icon = CSL_Factory:GetSpellName(bar.spellIds[index])
		debuffType = AuraCache:GetDebuffType(bar.spellIds[index])
	else
		icon = bar.spellIds[index]
	end
	
	cafr.icon:SetTexture(icon)
	if DebuffTypeColor[debuffType] then
		color = DebuffTypeColor[debuffType]
		cafr.slot:SetVertexColor(color.r, color.g, color.b)
		cafr.slot:Show()
	else
		if not bar.isbuffs[index] then
			cafr.slot:SetVertexColor(0.80, 0, 0)
			cafr.slot:Show()
		else
			cafr.slot:Hide()
		end
	end
	if (bar.reverses[index]) then
		cafr.icon:SetAlpha(1.0)
	else
		cafr.icon:SetAlpha(0.6)
	end
	if (bar.counts[index] > 1) then
		cafr.appl:SetText(bar.counts[index])
	else
		cafr.appl:SetText("")
	end
	if (bar.expirationTimes[index]) then
		local starttime, correction = bar.expirationTimes[index] - bar.durations[index], 0
		if starttime < 0 then
			correction = -starttime + 1
		end
		CooldownFrame_SetTimer(cafr.cool, starttime + correction, bar.durations[index] - correction, 1)
		-- cafr.cool:SetCooldown(starttime, bar.durations[index])
		cafr.cool:SetReverse(bar.reverses[index])
	end
	if (bar.reverses[index]) then
		cafr.cool:SetAlpha((self.db.char.cooldownalpha or 100) / 100)
	else
		cafr.cool:SetAlpha((self.db.char.cooldownalpha or 100) / 100 * 0.3)
	end
	if (bar.expirationTimes[index] > 0 and bar.durations[index] > 0) then
		cafr.cool:Show()
	else
		cafr.cool:Hide()
	end
	if tonumber(bar.spellIds[index]) then
		cafr.spellId = bar.spellIds[index]
	else
		cafr.spellId = nil
	end
	cafr.isbuff = bar.isbuffs[index]
	cafr.unitid = baroptions.unitid
	cafr.buffindex = bar.buffindexs[index]
	cafr.tooltipText = bar.tooltipTexts[index]

	cafr:Show()
end

function BdwAddon:DisplayAuras(barname)
	local baroptions = self.db.char.bars[barname]
	local bar = bars[barname]

	local position = 0
	local i

	for i=1, #bar.active do
		if (bar.active[i]) then
			position = position + 1
			self:CreateAuraIndicator(bar, baroptions, position, i)
		else
			break
		end
	end

	for i=position + 1, #bars[barname].afr do
		bars[barname].afr[i]:Hide()
	end  
end

function BdwAddon:EnableAllDragable(dragable)
	local k
	for k in pairs(bars) do
		self:EnableDragable(k, dragable)
	end
end 

function BdwAddon:EnableDragable(barname, dragable)
	local i

	for i=1, #bars[barname].afr do    
		bars[barname].afr[i]:SetMovable(dragable)
		-- bars[barname].afr[i]:EnableMouse(dragable)
	end  
	bars[barname].frame:SetMovable(dragable)
	bars[barname].frame:EnableMouse(dragable)
	if not dragable then
		bars[barname].frame.bck:Hide()
	else
		bars[barname].frame.bck:Show()
	end
end

function BdwAddon:UpdateFrameSize(barname)
	if not barname then return end
	local baropt = self.db.char.bars[barname]
	local bar = bars[barname]

	if bar and baropt then		
		bar.frame:SetWidth(baropt.barsize or 24)
		bar.frame:SetHeight(baropt.barsize or 24)

		for i = 1, #bar.afr do
			local cafr = bar.afr[i]
			cafr.appl:SetJustifyV(baropt.CountJustifyV or "BOTTOM")
			cafr.appl:SetJustifyH(baropt.CountJustifyH or "RIGHT")
			cafr.appl:SetFont(self.LSM:Fetch(self.LSM.MediaType.FONT, (baropt.CountFont or "Friz Quadrata TT")), baropt.CountFontSize or 11, baropt.CountFontFlags or "")
			cafr.appl:SetTextColor(baropt.CountR or 1, baropt.CountG or 1, baropt.CountB or 1, baropt.CountA or 1)
		end  
	end
end

function BdwAddon:AddAura(bar, index, spellId, count, duration, expirationTime, reverse, isbuff, buffindex, tooltipText)
	bar.active[index] = true
	bar.spellIds[index] = spellId
	bar.counts[index] = count
	bar.durations[index] = duration
	bar.expirationTimes[index] = expirationTime
	bar.reverses[index] = reverse
	bar.isbuffs[index] = isbuff
	bar.buffindexs[index] = buffindex
	bar.tooltipTexts[index] = tooltipText
end

function BdwAddon:GetMineAuras(barname)
	local name, icon, debuffType

	local isMine
	local isShort
	local showDebuff
	local showBuff
	local auragroup
	local group_count
	local tooltipText

	local baroptions = self.db.char.bars[barname]
	local bar = bars[barname]
	local key

	for k in pairs(bar.active) do bar.active[k] = false	end
	for k in pairs(bar.fauras) do bar.fauras[k] = 0	end
	for k in pairs(bar.auragroups) do bar.auragroups[k] = false	end

	key = 0

	if not UnitExists(baroptions.unitid) then return end

	local guid = UnitGUID(baroptions.unitid)
	local playerGUID = UnitGUID("player")
	local petGUID = UnitGUID("pet")
	local vehicleGUID = UnitGUID("vehicle")

	if (baroptions.showdebuffs) then
		local infotab = AuraCache:GetSortedAuras(guid, "DEBUFF")
		
		for i = 1, infotab.num do
			local k = infotab.sorted[i]
			local spellId, caster = infotab.spellId[k], infotab.caster[k]
			name = CSL_Factory:GetSpellName(spellId)

			isMine = caster == playerGUID or caster == petGUID or caster == vehicleGUID
			showDebuff = false

			if baroptions.showplayer then
				showDebuff = isMine
			end

			if baroptions.showcaster then
				auragroup = splnames.caster[spellId] 
				if auragroup then
					bar.auragroups[auragroup] = true
					if (bar.fauras[key] == 0 or bar.fauras[spellId] == nil) and not baroptions.onlyindi then
						showDebuff = true
					end
				end 
			end 

			if baroptions.showtank then
				auragroup = splnames.tanks[spellId]
				if auragroup then
					bar.auragroups[auragroup] = true 
					if (bar.fauras[key] == 0 or bar.fauras[spellId] == nil) and not baroptions.onlyindi then
						showDebuff = true
					end
				end
			end

			if baroptions.showalldebuffs then
				showDebuff = true
			end

			if showDebuff then
				if string.find(baroptions.ignorelist, name) then
					showDebuff = false
				end
			else
				if string.find(baroptions.includelist, name) then
					showDebuff = true
				end 
			end
  
			if showDebuff then
				key = key + 1
															
				self:AddAura(bar, key, spellId, infotab.count[k], infotab.duration[k], infotab.expirationTime[k], true, false, k)

				bar.fauras[spellId] = key
			end
		end
	end
		

	if baroptions.showbuffs then
		local infotab = AuraCache:GetSortedAuras(guid, "BUFF")
		
		for i = 1, infotab.num do
			local k = infotab.sorted[i]
			local spellId, caster = infotab.spellId[k], infotab.caster[k]
			name, icon = CSL_Factory:GetSpellName(spellId)

			if (baroptions.duration or 0) > 0 then
				isShort = (infotab.duration[k] <= baroptions.duration)
			else
				isShort = true
			end
			if not baroptions.showauras then
				if infotab.duration[k] == 0 then
					isShort = false
				end 
			end

			if baroptions.showplayerbuff then
				isMine = caster == playerGUID or caster == petGUID or caster == vehicleGUID
			else
				isMine = false
			end

			if baroptions.showenemybuff then
				if UnitCanAttack("player", baroptions.unitid) and not UnitIsPlayer(baroptions.unitid) then
					isMine = true
				end
			end

			if baroptions.showallbuffs then
				isMine = true
			end

			showBuff = isMine and isShort

			if showBuff then    
				if string.find(baroptions.ignorelist, name) then
					showBuff = false
				end
			else
				if string.find(baroptions.includelist, name) then
					showBuff = true
				end  
			end

			if showBuff then    
				key = key + 1
					
				self:AddAura(bar, key, spellId, infotab.count[k], infotab.duration[k], infotab.expirationTime[k], true, true, k)				                         
			end
		end
	end

	if baroptions.showcombo then   
		local comboPoints = 0
		if (UnitInVehicle("player")) then
			comboPoints = GetComboPoints("vehicle", baroptions.unitid)
		else
			comboPoints = GetComboPoints("player", baroptions.unitid)
		end

		if (comboPoints > 0) then
			key = key + 1
			self:AddAura(bar, key, combo_icon[comboPoints], comboPoints, 0, 0, true, false, nil)
		end
	end

	if (baroptions.onlyindi and UnitCanAttack("player", baroptions.unitid)) then
		if (baroptions.showcaster) then
			group_count = 0
			tooltipText = nil

			for iIterator=1, 2 do
				if (bar.auragroups[iIterator]) then
					group_count = group_count + 1
				elseif (agnames[iIterator] ~= nil) then
					if (tooltipText ~= nil) then
						tooltipText = tooltipText .. "\n" .. agnames[iIterator]
					else
						tooltipText = agnames[iIterator]
					end
				end
			end

			if (group_count == 0) then
				icon = red_icon
			elseif (group_count < 2) then
				icon = yellow_icon
			else
				icon = green_icon
			end 
        
			key = key + 1
			self:AddAura(bar, key, icon, 1, 0, 0, true, false, nil, tooltipText)
		end

		if (baroptions.showtank) then
			group_count = 0
			tooltipText = nil

			for iIterator=5, 9 do
				if (bar.auragroups[iIterator]) then
					group_count = group_count + 1
				elseif (agnames[iIterator] ~= nil) then
					if (tooltipText ~= nil) then
						tooltipText = tooltipText .. "\n" .. agnames[iIterator]
					else
						tooltipText = agnames[iIterator]
					end
				end
			end

			if (group_count == 0) then
				icon = red_icon
			elseif (group_count <= 2) then
				icon = yellow_icon
			else
				icon = green_icon
			end 

			key = key + 1
			self:AddAura(bar, key, icon, 1, 0, 0, true, false, nil, tooltipText)
		end
	end
end

function BdwAddon:AuraCacheNotifier(unitId)
	for k in pairs(bars) do
		if unitId == self.db.char.bars[k].unitid then
			self:GetMineAuras(k)
			self:DisplayAuras(k)
		end
	end
end

function BdwAddon:PLAYER_TARGET_CHANGED(msg)
	AuraCache:UpdateUnitAura("target")
	for k in pairs(bars) do
		if self.db.char.bars[k].unitid == "target" then
			self:GetMineAuras(k)
			self:DisplayAuras(k)
		end
	end
end

function BdwAddon:PLAYER_FOCUS_CHANGED()
	AuraCache:UpdateUnitAura("focus")
	for k in pairs(bars) do
		if self.db.char.bars[k].unitid == "focus" then
			self:GetMineAuras(k)
			self:DisplayAuras(k)
		end
	end
end

function BdwAddon:UNIT_PET(arg1)
	if (UnitIsUnit(arg1, "player")) then
		for k in pairs(bars) do
			if self.db.char.bars[k].unitid == "pet" then
				self:GetMineAuras(k)
				self:DisplayAuras(k)
			end
		end
	end
end

function BdwAddon:UNIT_COMBO_POINTS(arg1)
	for k in pairs(bars) do
		if self.db.char.bars[k].showcombo then
			self:GetMineAuras(k)
			self:DisplayAuras(k)
		end
	end
end


function BdwAddon:UPDATE_MOUSEOVER_UNIT()
	for k in pairs(bars) do
		if self.db.char.bars[k].unitid == "mouseover" then
			self:GetMineAuras(k)
			self:DisplayAuras(k)
		end
	end
end

function BdwAddon:OnSlash(msg)
	LibStub("AceConfigDialog-3.0"):Open("BDW")
end

function hook_SetUnitAura(self, unit, index, filter)
	local casterName
	local _, _, _, _, _, _, _, caster, isStealable, _, spellID = UnitAura(unit, index, filter)

	if caster then
		casterName = UnitName(caster)
	end

	if isStealable then
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 0, 1, 0)
		else
			self:AddLine(tostring(spellID), 0, 1, 0)
		end
	else
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 1, 1, 0)
		else
			self:AddLine(tostring(spellID), 1, 1, 0)
		end
	end
    self:Show()
end

function hook_SetUnitBuff(self, unit, index, filter) 
	local casterName
	local _, _, _, _, _, _, _, caster, isStealable, _, spellID = UnitBuff(unit, index, filter)

	if caster then
		casterName = UnitName(caster)
	end

	if isStealable then
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 0, 1, 0)
		else
			self:AddLine(tostring(spellID), 0, 1, 0)
		end
	else
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 1, 1, 0)
		else
			self:AddLine(tostring(spellID), 1, 1, 0)
		end
	end
    self:Show()
end

function hook_SetUnitDebuff(self, unit, index, filter) 
	local casterName
	local _, _, _, _, _, _, _, caster, isStealable, _, spellID = UnitDebuff(unit, index, filter)

	if caster then
		casterName = UnitName(caster)
	end

	if isStealable then
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 0, 1, 0)
		else
			self:AddLine(tostring(spellID), 0, 1, 0)
		end
	else
		if casterName then
			self:AddDoubleLine(casterName, tostring(spellID), 1, 1, 0)
		else
			self:AddLine(tostring(spellID), 1, 1, 0)
		end
	end
	self:Show()
end
